/*
 * Input.java
 *
 * Created on August 9, 2000, 2:59 PM
 */
 
package saaps.ann;

import java.util.Vector;

/** 
 * To run a neural network one must specify the input data and
 * then one receives the output data. This class is the input or
 * output data.
 *
 * @author  Peter Wintoft
 * @version 1.0 2000-08-09
 * @see DataCreator
 */
public class Data extends Object {
  //Number of examples
  private int numOfExamples;
  //Number of input or output connections.
  private int numOfConnections;
  //The data. The data array is arranged so that the first
  //index is the connection index and the second index the
  //example index.
  private double[][] data;
  
  /** Creates new Input */
  public Data() {
  }
  
  /**
  Create a new empty <code>Data</code> object with specified
  number of connections and examples.
  <p>
  Each connection represents an input or an output. E.g., for
  a network with 10 inputs and 3 outputs would need a <code>Data</code>
  object with 10 connections for its input. The output from
  the network will be a <code>Data</code> object with 3 connections.
  
  @param  conn  The number of connections
  @param  exam  The number of examples
  */
  public Data(int conn, int exam)
  {
    numOfConnections = conn;
    numOfExamples = exam;
    data = new double[numOfConnections][numOfExamples];
//    for (int c=0;c<numOfConnections;c++)
//      data[c] = new double[numOfExamples];
  }
  
  /**
  Create a new <code>Data</code> object from an array of an array of
  <code>double</code>.
  <p>
  It is assumed that the first index of the array corresponds to the
  connections and the second index corresponds to the examples.
  
  @param  d The data
  */
  public Data(double[][] d)
  {
    numOfConnections = d.length;
    numOfExamples = d[0].length;
    data = new double[numOfConnections][numOfExamples];
    for (int conn=0;conn<numOfConnections;conn++)
      for (int exam=0;exam<numOfExamples;exam++)
        data[conn][exam] = d[conn][exam];
  }
  
  public Data(double[] d)
  {
    data = new double[1][];
    data[0] = d;
    numOfConnections = 1;
    numOfExamples = data[0].length;
  }
  
  public Data(Data d)
  {
    data = d.get();
    numOfConnections = data.length;
    numOfExamples = data[0].length;
  }
  
  public int getNumOfExamples()
  {
    return numOfExamples;
  }
  
  public int getNumOfConnections()
  {
    return numOfConnections;
  }
  
  public double[][] get()
  {
    return data;
  }
  
  public double[] getConnection(int c)
  {
    return data[c];
  }
  
  public double get(int conn, int exam)
  {
    return data[conn][exam];
  }
  
  public void set(int conn, int exam, double val)
  {
    data[conn][exam] = val;
  }
  
  public void setConnection(int conn, double[] d)
  {
    data[conn] = d;
  }
  
  public Data weightedSum(Weights weights)
  {
      double[] w = weights.getWeights();
      double b = weights.getBias();
      double sum[] = new double[numOfExamples];
      for (int exam=0;exam<numOfExamples;exam++)
      {
        sum[exam] = b;
        for (int conn=0;conn<numOfConnections;conn++)
          sum[exam] += data[conn][exam] * w[conn];
      }
      return new Data(sum);
  }
  
  public void print()
  {
    for(int conn=0;conn<numOfConnections;conn++)
    {
      System.out.print("Connection " + conn + ": ");
      for (int exam=0;exam<numOfExamples;exam++)
        System.out.print(data[conn][exam] + " ");
      System.out.println();
    }
  }
 
}