/*
 * TimeSeriesToDataMap.java
 *
 * Created on September 13, 2000, 2:21 PM
 */
 
package saaps.ann;

import saaps.util.TimeSeries;
import java.io.Serializable;

/** 
 * The <code>TimeSeriesToDataMap</code> is used to convert a <code>TimeSeries</code> object
 *  into a <code>Data</code> object to be used as input to a neural network. Two things are
 *  made with the mapper: normalization and transformation.
 *  <p>
 *  The mapping information is given in two integer arrays connecting the elements in the
 *  <code>TimeSeries</code> object to the <code>Data</code> object.
 *  <p>
 *  The first array, the <code>time</code> array, describes the time delay or time shifts.
 *  The second array, the <code>field</code> array, describes what data field should be used.
 *  The lengths of <code>time</code> and <code>field</code> must be the same. The number of
 *  connections (inputs) of the resulting <code>Data</code> object will equal the length of
 *  <code>time</code> (and <code>field</code>). The number of examples in the <code>Data</code>
 *  object will equal the number of examples in the <code>TimeSeries</code> object minus the
 *  length of <code>time</code> plus one.
 *  <p>
 *  The easiest way to determine <code>time</code> and <code>field</code> is to consider what the
 *  first example in the <code>Data</code> object should be. Suppose that the first three
 *  observations of data fields 2 and 3 of the <code>TimeSeries</code> object should make up the
 *  first example in the <code>Data</code> object. Then we will have <code>time=[0,1,2,0,1,2]</code>
 *  and <code>field=[2,2,2,3,3,3]</code>. The same mapping is then shifted one time step at a time
 *  through the <code>TimeSeries</code> object to generate new examples in the <code>Data</code> object.
 *
 * @author  Peter Wintoft, 2000-11-08
 * @version 1.0
 */
public class TimeSeriesToDataMap extends Object implements Serializable {

  //The time mapping.
  private int[] time;
  //The field mapping.
  private int[] field;
  //Used to determine the number of examples.
  private int timeMax;
  
  /** Creates new TimeSeriesToDataMap */
  public TimeSeriesToDataMap() {
  }
  
  /**
  * Create the mapping object from two integer arrays of the same length.
  *
  * @param  times The time mapping
  * @param  fields  The data field mapping
  */
  public TimeSeriesToDataMap(int[] times, int[] fields)
  {
//    time = new int[times.length];
//    field = new int[fields.length];
    time = (int[]) times.clone();
    field = (int[]) fields.clone();
    timeMax = time[0];
    for (int i=1;i<time.length;i++)
      if (time[i]>timeMax)
        timeMax = time[i];
  }
  
  /**
  * Convert the <code>TimeSeries</code> object into a <code>Data</code> object
  * according to the specified mapping.
  *
  * @param  ts  The time series to be converted
  * @return The <code>Data</code> object
  */
  public Data convert(TimeSeries ts)
  {
    int numOfConnections = time.length;
    int numOfExamples = ts.getLength() - timeMax;
    Data data = new Data(numOfConnections,numOfExamples);
    for (int conn=0;conn<numOfConnections;conn++)
      for (int exam=0;exam<numOfExamples;exam++)
        data.set(conn,exam,ts.getData(exam+time[conn],field[conn]));
    return data;
  }
}