/*
 * DataCreator.java
 *
 * Created on August 10, 2000, 1:38 PM
 */
 
package saaps.ann;

import java.io.BufferedReader;
import java.io.FileNotFoundException;
import java.io.FileReader;
import java.io.IOException;
import java.io.StreamTokenizer;
import java.util.Vector;

/** 
 * The <code>WeightsCreator</code> class is used to read weight data
 *  from an ASCII file. 
 *
 * @author    Peter Wintoft 2000-11-07
 * @version   1.0
 */
public class WeightsCreator extends Object {

  /**
  * Read the weights from an ASCII file and create an array of
  * <code>Weights</code> objects. The ASCII file must have the
  * following format:
  * <table cols=5 width=400>
  * <code>
  * <tr>
  * <td> m </td> <td> n </td> <td> </td> <td> </td> <td> </td>
  * </tr>
  * <tr>
  * <td> w(1,1) </td> <td> w(1,2) </td> <td> ... </td> <td> w(1,n) </td> <td> b(1) </td>
  * </tr>
  * <tr>
  * <td> w(2,1) </td> <td> w(2,2) </td> <td> ... </td> <td> w(2,n) </td> <td> b(2) </td>
  * </tr>
  * <tr>
  * <td> ... </td> <td> ... </td> <td> ... </td> <td> ... </td> <td> ... </td>
  * </tr>
  * <tr>
  * <td> w(m,1) </td> <td> w(m,2) </td> <td> ... </td> <td> w(m,n) </td> <td> b(m) </td>
  * </tr>
  * </code>
  * </table>
  * where m is the number of neurons and n is the number of inputs. The w(i,j) is the weight
  * connecting input j to neuron i, and b(i) is the bias of neuron i. The order of the data
  * in the ASCII file must be the same as above, however, the data need not be separated by
  * new-lines, any white-space characters will work.
  *
  * @param  fileName  The file name of the ASCII weights file.
  * @return An array of Weights objects. The number of Weights objects equals the number
  * of neurons, and the number of weights in each Weights object equals the number of inputs.
  * @see  Weights
  */
  public static Weights[] read(String fileName)
  {
    Weights[] weights = null;
    try
    {
      //Open file and create a stream tokenizer.
      FileReader fr = new FileReader(fileName);
      BufferedReader br = new BufferedReader(fr);
      StreamTokenizer st = new StreamTokenizer(br);
      //Get the number of neurons.
      st.nextToken();
      int numOfNeurons = (int) st.nval;
      //Get the number of inputs.
      st.nextToken();
      int numOfInputs = (int) st.nval;
      //Initialize the weights.
      weights = new Weights[numOfNeurons];
      double[] oneWeight = new double[numOfInputs];
      double oneBias;
      //Read data.
      for (int n=0;n<numOfNeurons;n++)
      {
        for (int i=0;i<numOfInputs;i++)
        {
          st.nextToken();
          oneWeight[i] = st.nval;
        }
        st.nextToken();
        oneBias = st.nval;
        weights[n] = new Weights(oneWeight,oneBias);
      }
      fr.close();
    }
    catch(FileNotFoundException fne)
    {
      System.err.println(fne);
    }
    catch(IOException ioe)
    {
    }
    return weights;
  }
  
  /**
  * Read the weights from an ASCII file specifying the number of inputs.
  *
  * @deprecated This method is replaced by read(String).
  */
  public static Weights[] read(String fileName,int numOfInputs)
  {
    //Weights to one neuron.
    double[] oneWeight = new double[numOfInputs];
    double oneBias;
    //The weights to all neurons.
    Vector allWeights = new Vector(10);
    Vector allBiases = new Vector(10);
    try
    {
      //Open file and create a stream tokenizer.
      FileReader fr = new FileReader(fileName);
      BufferedReader br = new BufferedReader(fr);
      StreamTokenizer st = new StreamTokenizer(br);
      //Read data until end of file.
      int inp = 0;
      while (st.ttype != StreamTokenizer.TT_EOF)
      {
        st.nextToken();
        if (inp<numOfInputs)
          oneWeight[inp++] = st.nval;
        else
        {
          oneBias = st.nval;
          allWeights.addElement(oneWeight);
          allBiases.addElement(new Double(oneBias));
          inp = 0;
          oneWeight = new double[numOfInputs];
        }
      }
      fr.close();
    }
    catch(FileNotFoundException fne)
    {
      System.err.println(fne);
    }
    catch(IOException ioe)
    {
    }
    allWeights.trimToSize();
    allBiases.trimToSize();
    int numOfNeurons = allWeights.size();
    Weights[] weights = new Weights[numOfNeurons];
    for (int neuron=0;neuron<numOfNeurons;neuron++)
    {
      Double b = (Double) allBiases.elementAt(neuron);
      weights[neuron] = new Weights((double[]) allWeights.elementAt(neuron), b.doubleValue());
    }
    return weights;
  }
}