package saaps.dbt;

import java.io.BufferedReader;
import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileReader;
import java.io.IOException;
import java.util.Calendar;
import java.util.TimeZone;
import java.util.Vector;

import saaps.dbt.Kp;
import saaps.dbt.KpDB;

/**
The <code>BuildKp</code> application will create the Kp database
from the text files in the directory <code>kpdir</code>.
The text files have been downloaded from
ftp://ftp.ngdc.noaa.gov/STP/GEOMAGNETIC_DATA/INDICES/KP_AP/ . The
downloaded data starts from 1970. The application is run by typing
<br>
<code>java BuildKp &lt kpdir& gt</code>
<br>
where <code>kpdir</code> is the directory of the Kp data.

@author Peter Wintoft
@version 1.0
*/
public class BuildKp {

    //The base directory of the Kp data.
    private String kpDir;
    
    //A vector containing one year of Kp data.
    private Vector kpOneYear;
    
    //A calendar object to do time calculations.
    private Calendar cal = Calendar.getInstance(TimeZone.getTimeZone("GMT"));

    ///////////////////////////////
    //Constructors
    ///////////////////////////////

    /**
    Create an object by specifying the base directory
    of the Kp data.
    
    @param kpDir The base directory
    */
    public BuildKp(String kpDir) {
        //Check that the directory name ends
        //with a "/", else add one.
        if (kpDir.endsWith("/")) {
            this.kpDir = new String(kpDir);
        }
        else {
            this.kpDir = new String(kpDir + "/");
        }
    }
    
    ///////////////////////////////
    //Public methods
    ///////////////////////////////
    
    /**
    Run the application.
    */
    public void run() {
        System.out.println("Using directory " + this.kpDir);
        //Find the first year of data.
        int year = this.getFirstYear();
        System.out.println("Will read year " + year + " ...");
        //Read one year of data at a time until no more data.
        while (this.readOneYearOfData(year++)) {
            System.out.println("Reading and storing year " + ((int) year-1));
            //Store the data in the database.
            this.storeOneYearData();
        }
    }
    
    /**
    Get the year of the first Kp file.
    
    @returns The year.
    */
    public int getFirstYear() {
        //The first year.
        int y = 0;
        //List the files in the directory.
        try {
            File file = new File(this.kpDir);
            String[] fileName = file.list();
            y = Integer.parseInt(fileName[0]);
        }
        catch (SecurityException e) {
            System.err.println(e);
            System.exit(0);
        }
        catch (NullPointerException e) {
            System.err.println(e);
            System.exit(0);
        }
        catch (NumberFormatException e) {
            System.err.println(e);
            System.exit(0);
        }
        return y;
    }
    
    /**
    Read one year of data for the specified year. If no file
    exist for that year the method returns <code>false</code>.
    
    @param year The year of the data to be read.
    @returns <code>True</code> if the file for the given year
    exists, else <code>False</code>.
    */
    public boolean readOneYearOfData(int year) {
        //status indicates whether the file could be opened or not.
        boolean status = true;
        //Initialize to hold 366 days of data.
        this.kpOneYear = new Vector(366*8);
        //Create the file name.
        String fileName = new String(this.kpDir + year);
        System.out.println("Reading file " + fileName + " ...");
        try {
            //Open the file.
            BufferedReader buf = new BufferedReader(new FileReader(fileName));
            //Read one line of text at a time until end of file.
            String textLine = null;
            while ((textLine = buf.readLine()) != null) {
                //Parse the text line and create a eight Kp objects for a day
                //and add to the Kp vector.
//                Kp[] kp = this.parseKpData(textLine);
                this.parseKpData(textLine);
//                //Add to the vector.
//                for (int i=0;i<kp.length;i++) {
//                    this.kpOneYear.add(kp[i]);
//                }
            }
            //Trim the size of the vector.
            this.kpOneYear.trimToSize();
        }
        catch (FileNotFoundException e) {
            status = false;
        }
        catch (IOException e) {
            System.err.println(e);
            System.exit(0);
        }
        return status;
    }
    
    /**
    Parse a line of text and add Kp objects to the Kp vector.
    
    @param text The string containing the Kp data.
//    @returns The eight Kp objects for one day.
    */
//    public Kp[] parseKpData(String text) {
    public void parseKpData(String text) {
        //Initialize.
        cal.clear();
        try {
            //Read the two digit year.
            //If year is less than 32 then add 2000, else add 1900;
            int year = Integer.parseInt(text.substring(0,2));
            if (year<32) {
                year += 2000;
            }
            else {
                year += 1900;
            }
            //Read the month.
            //Subtract 1 as Java months starts at 0.
            int month = Integer.parseInt(text.substring(2,4).trim()) - 1;
            //Read the day.
            int day = Integer.parseInt(text.substring(4,6).trim());
            //Set the date.
            this.cal.set(Calendar.YEAR,year);
            this.cal.set(Calendar.MONTH,month);
            this.cal.set(Calendar.DATE,day);
            //Read the eight Kp values.
            //Divide by 10 as the values in the file are Kp*10.
            //The first Kp value is at 00UT, the second at 03UT, and so on.
            for(int i=0;i<8;i++) {
                int hour = 3*i;
                this.cal.set(Calendar.HOUR_OF_DAY,hour);
                //Get the Kp position.
                int i1 = 12 + 2*i;
                int i2 = 14 + 2*i;
                //Get the Kp*10 value.
                double kp10 = Double.parseDouble(text.substring(i1,i2));
                //Create the Kp object.
//                kp[i] = new Kp(this.cal.getTime(),kp10/10);
                Kp kp = new Kp(this.cal.getTime(),kp10/10);
                //Will be replaced by ...
                //kp[i] = new Kp(this.cal.getTime(),{kp10/10,1});
                //Add to the Kp vector.
                this.kpOneYear.add(kp);
//System.out.println(kp);
            }
        }
        catch (NumberFormatException e) {
            System.err.println(e);
            System.err.println(text);
            System.exit(0);
        }
//        return kp;
    }
    
    /**
    Store one year of data into the SAAPS database.
    */
    public void storeOneYearData() {
        //Create the database connection.
        Database db = Database.getInstance("KpDB");
        //Store into to the database.
        db.store(this.kpOneYear);
        try {
            //Close.
            db.close();
        }
        catch (IOException e) {
            System.err.println(e);
            System.exit(0);
        }
    }

    ///////////////////////////////
    //Main
    ///////////////////////////////

    /**
    The uses one argument to locate the directory
    where the Kp text files are located. The file
    names are assumed to be named according to the
    year of the data, e.g. 1970, 1971, and so on.
    
    @param args A string array with the Kp directory
    at the first element.
    */
    public static void main(String[] args) {
        //Check that the number of arguments is correct.
        if (args.length!=1) {
            System.err.println("Usage: BuildKp <Kp directory>");
            System.exit(0);
        }
        //Initialize.
        BuildKp bkp = new BuildKp(args[0]);
        //Run the application.
        bkp.run();
    }
}