package saaps.dbt;

import java.text.DateFormat;
import java.text.NumberFormat;

import java.util.Calendar;
import java.util.Date;
import java.util.Locale;
import java.util.TimeZone;
import java.io.Serializable;
import java.io.ByteArrayOutputStream;
import java.io.DataOutputStream;
import java.io.IOException;

import java.util.Vector;

/**
* A class for GOES-08 particle data.
*
* The class contains the date, and electron and proton
* fluxes for the following energies:
*   P > 1 MeV
*   P > 5 MeV
*   P >10 MeV
*   P >30 MeV
*   P >50 MeV
*   P>100 MeV
*   E>0.6 MeV
*   E>2.0 MeV
*   E>4.0 MeV
* It also contain the longitude position of the satellite.
*
* @author Peter Wintoft
* @version 1.0 1999-12-22
*/
public class GOES08part extends DatabaseObject implements Serializable
{
  // Fields
  private static int numberOfIndexes = 10;
  private Date d;
  private double p1, p5, p10, p30, p50, p100, e06, e2, e4, longitude;

  // Constructors
  public GOES08part()
  {
  }
  /**
   * This constructor is used if you want an "empty" object with this date.
   */
  public GOES08part(Date D) {
    d = D;
    p1 = Double.NaN;
    p5 = Double.NaN;
    p10 = Double.NaN;
    p30 = Double.NaN;
    p50 = Double.NaN;
    p100 = Double.NaN;
    e06 = Double.NaN;
    e2 = Double.NaN;
    e4 = Double.NaN;
    longitude = Double.NaN;
  }

  public GOES08part(Date D, double P1, double P5, double P10, double P30, double P50, double P100, double E06, double E2, double E4, double LG)
  {
    d = D;
    p1 = P1;
    p5 = P5;
    p10 = P10;
    p30 = P30;
    p50 = P50;
    p100 = P100;
    e06 = E06;
    e2 = E2;
    e4 = E4;
    longitude = LG;
  }
  public GOES08part(long L, double P1, double P5, double P10, double P30, double P50, double P100, double E06, double E2, double E4, double LG)
  {
    this(new Date(L),P1,P5,P10,P30,P50,P100,E06,E2,E4,LG);
  }

  public GOES08part(Date D, double[] data) {
    this(D,data[0],data[1],data[2],data[3],data[4],data[5],data[6],data[7],data[8],data[9]);
  }

  // Methods
  public double getP1()
  {
    return p1;
  }
  public double getP5()
  {
    return p5;
  }
  public double getP10()
  {
    return p10;
  }
  public double getP30()
  {
    return p30;
  }
  public double getP50()
  {
    return p50;
  }
  public double getP100()
  {
    return p100;
  }
  public double getE06()
  {
    return e06;
  }
  public double getE2()
  {
    return e2;
  }
  public double getE4()
  {
    return e4;
  }
  public double getLongitude()
  {
    return longitude;
  }

  //Implemented for DatabaseObject
  public Date getDate()
  {
    return d;
  }

  // This method is the generic form of accessing data.
  public Vector getData()
  {
    Vector vec = new Vector(numberOfIndexes);
    vec.addElement(new Double(p1));
    vec.addElement(new Double(p5));
    vec.addElement(new Double(p10));
    vec.addElement(new Double(p30));
    vec.addElement(new Double(p50));
    vec.addElement(new Double(p100));
    vec.addElement(new Double(e06));
    vec.addElement(new Double(e2));
    vec.addElement(new Double(e4));
    vec.addElement(new Double(longitude));
    return vec;
  }

  /**
   * This method returns the size of the data that is to be written to file
   */
  public long getDataSize() throws IOException {

    ByteArrayOutputStream baos = new ByteArrayOutputStream();  
    DataOutputStream    dos  = new DataOutputStream(baos);  
    dos.writeLong(d.getTime());
    dos.writeDouble(p1);
    dos.writeDouble(p5);
    dos.writeDouble(p10);
    dos.writeDouble(p30);
    dos.writeDouble(p50);
    dos.writeDouble(p100);
    dos.writeDouble(e06);
    dos.writeDouble(e2);
    dos.writeDouble(e4);
    dos.writeDouble(longitude);

    dos.flush();  
    long size = baos.size();  // now you know how large ....
    return size;
  }

  public void setDate(Date D)
  {
    d = D;
  }
  public void setDate(long L)
  {
    d = new Date(L);
  }
  public void setP1(double P1)
  {
    p1 = P1;
  }
  public void setP5(double P5)
  {
    p5 = P5;
  }
  public void setP10(double P10)
  {
    p10 = P10;
  }
  public void setP30(double P30)
  {
    p30 = P30;
  }
  public void setP50(double P50)
  {
    p50 = P50;
  }
  public void setP100(double P100)
  {
    p100 = P100;
  }
  public void setE06(double E06)
  {
    e06 = E06;
  }
  public void setE2(double E2)
  {
    e2 = E2;
  }
  public void setE4(double E4)
  {
    e4 = E4;
  }
  public void setLongitude(double LG)
  {
    longitude = LG;
  }

  //Implemented for DatabaseObject getDate
  public int getNumberOfDataIndexes() {
    return (numberOfIndexes);
  }

  //Implemented for DatabaseObject
  public double getData(int index) {
    //Date is not considered a data value
    switch (index) {
    case 0 :
      return(p1);
    case 1 :
      return(p5);
    case 2 :
      return(p10);
    case 3 :
      return(p30);
    case 4 :
      return(p50);
    case 5 :
      return(p100);
    case 6 :
      return(e06);
    case 7 :
      return(e2);
    case 8 :
      return(e4);
    case 9 :
      return(longitude);
    default :
      System.out.println("Index out of bounds for GOES-08");
      return (Double.NaN);
    }
  }

  public String toString() {
//     // Set the date format.
//     DateFormat df = DateFormat.getDateTimeInstance(DateFormat.DEFAULT,DateFormat.DEFAULT,new Locale("sv","SE")); //en UK
//     df.setTimeZone(TimeZone.getTimeZone("UTC"));

    // Find the selected dates.
    Calendar cal = Calendar.getInstance(TimeZone.getTimeZone("UTC"));
    cal.clear(cal.MILLISECOND);
    cal.setTime(d);

    NumberFormat nf = NumberFormat.getInstance();
    nf.setMinimumIntegerDigits(2);

    return ( cal.get(Calendar.YEAR) + " " + nf.format(cal.get(Calendar.MONTH)+1) + " " + nf.format(cal.get(Calendar.DAY_OF_MONTH)) + " " +
             nf.format(cal.get(Calendar.HOUR_OF_DAY)) + " " + nf.format(cal.get(Calendar.MINUTE)) + " " + nf.format(cal.get(Calendar.SECOND)) + " " +
             Double.toString(p1) + " " + Double.toString(p5) + " " + Double.toString(p10) + " " + Double.toString(p30) + " " +
             Double.toString(p50) + " " + Double.toString(p100) + " " +
             Double.toString(e06) + " " + Double.toString(e2) + " " + Double.toString(e4) + " " +
             Double.toString(longitude) );

  }

}
