package saaps.dbt;

import java.io.*;
import java.net.*;
import java.text.*;
import java.util.*;

import saaps.util.*;

/**
* LatestGOES10xray downloads the latest GOES10 xray data and position.
* The fields are:
* short (0.05-0.4 nm) and long (0.1-0.8 nm) X-rays, and longitude.
* FTP address: ftp://ftp.sec.noaa.gov/pub/lists/xray/G10xr_5m.txt
*
*@author Peter Wintoft
*@version 1.0 2000-08-03
*/
public class LatestGOES10xray extends Object{
  private static double dataGapValue = -1.00e+05;
  public static void main(String[] args)
  {
    String ftpSrcStr="ftp://ftp.sec.noaa.gov/pub/lists/xray/G10xr_5m.txt";
    DateFormat df;
    Date d;
    Vector goesv = null;
    double longitude = Double.NaN;
    try
    {
      // Open connection
      URL url = new URL(ftpSrcStr);
      URLConnection urlConn = url.openConnection();
      // Input stream.
      BufferedReader streamIn = new BufferedReader(
                                  new InputStreamReader(
                                    urlConn.getInputStream()));
      String textLine = "";
      textLine = streamIn.readLine();
      // Go through the file for all lines starting with
      // "#", ":".
//      while (!textLine.startsWith("1") && !textLine.startsWith("2"))
      while (textLine.charAt(0)=='#' || textLine.charAt(0)==':')
      {
//        System.out.println("Skipping: " + textLine);
        // Get the longitude value.
        if (textLine.startsWith("# Location:"))
          longitude = Double.valueOf(textLine.substring(13,16)).doubleValue();
        textLine = streamIn.readLine();
      }
      // Create a Vector of GOES10part objects.
      goesv = new Vector(24);
      while ((textLine = streamIn.readLine()) != null)
      {
//        System.out.println("Reading: " + textLine);
        // Tokenize the string.
        StringTokenizer st = new StringTokenizer(textLine);
        // Create the date string.
        String dateStr = st.nextToken() + "-" +    // Year
                          st.nextToken() + "-" +  // Month
                          st.nextToken() + " ";    // Day
        String timeStr = st.nextToken();          // Hour and minute
        // Insert a ":" between hour and minute.
        timeStr = timeStr.substring(0,2) + ":" + timeStr.substring(2,4);
        dateStr = dateStr + timeStr;
        // Read the date using - as the field separator.
//        String dateStr = textLine.substring(0,4) + "-" +
//                           textLine.substring(5,7) + "-" +
//                           textLine.substring(8,10) + " " +
//                           textLine.substring(11,13) + ":" +
//                           textLine.substring(13,15);
        // Use UTC time and get the Date object.
        df = DateFormat.getDateTimeInstance(DateFormat.SHORT,DateFormat.SHORT,new Locale("sv","SE"));
        df.setTimeZone(TimeZone.getTimeZone("UTC"));
        d = df.parse(dateStr);
        // Skip over until P>1 field.
        st.nextToken(); //Julian day.
        st.nextToken(); //Seconds of the day.
        // Read short and long xrays, and longitude.
        double shortXR = Double.valueOf(st.nextToken()).doubleValue();
        double longXR = Double.valueOf(st.nextToken()).doubleValue();
        // Replace data gaps with NaNs.
        if (shortXR == dataGapValue)
          shortXR = Double.NaN;
        if (longXR == dataGapValue)
          longXR = Double.NaN;
        // Add data to vector.
        goesv.addElement(new GOES10xray(d,shortXR,longXR,longitude));
      }
      // Trim the vector.
      goesv.trimToSize();
      // Close input stream.
      streamIn.close();
      // Store the data in the database
      Database db = Database.getInstance("GOES10xrayDB");

      ObjectTimeSeries ts = new ObjectTimeSeries(goesv);

      if(!db.isSet())
	System.out.println("MetaData for GOES10xrayDB is unavailable \nWill try to create it.");
			
      System.out.println("Got instance of DB will do store");
      db.store(ts);

    }
    catch (IOException e)
    {
      System.err.println("URL Connection error: " + e);
    }
    catch (NumberFormatException e)
    {
      System.err.println("Number format error: " + e);
    }
    catch (ParseException e)
    {
      System.err.println("Parse error: " + e);
    }
    catch (IllegalTimeSeriesException e) {
      System.err.println(e);
    }
    catch (Exception e) {
        System.err.println(e);
    }
  }
}
