/*
 * Anom003Model.java
 *
 * Created on April 2, 2001, 3:37 PM
 */

package saaps.sapm;

import java.io.IOException;
import java.util.Date;

import saaps.ann.*;
import saaps.ann.mlff.*;
//import saaps.dbt.Database;
import saaps.dbt.ODIDatabase;
import saaps.util.TimeSeries;

/**
 * The <code>Anom003Model</code> implements the neural network
 * for nowcasting of satellite S003 anomalies on a daily basis.
 * The input to the network are Kp summed over 24 hour periods
 * extending over eights days.
 * <p>
 * The output from the model is a <code>TimeSeries</code> object
 * that contain one data field which gives the probability in percent
 * that an anomaly will occur.
 * <p>
 * The time stamp of the latest predicted value will always be
 * the time of the last Kp value minus 24 hours. A prediction like
 * <br><br>
 * <code>
 * 2001 04 01 18 00 00 1 80
 * </code>
 * <br><br>
 * has used Kp values over the period 2001-04-01 18:00 to 2001-04-02
 * 18:00, and the predicted anomaly (1) with 80% probability is valid
 * over the same time period.
 *
 * @author  Peter Wintoft
 * @version 1.0
 */
public class Anom003Model extends saaps.util.Model {
    
    //The neural network.
    private Network network;
    
    //The mapping from the time series object to the data object.
    //The network uses the last seven days of sum_Kp.
    //Thus, the time array goes from 0 to 6 and the
    //field array is zero.
    private int[] times =  {0,1,2,3,4,5,6};
    private int[] fields = {0,0,0,0,0,0,0};
    private TimeSeriesToDataMap tsMap = new TimeSeriesToDataMap(times,fields);
    
    //Normalization of the input data. There are five inputs.
    private double[] inMin = {    0,   0,   0,   0,   0,   0,   0};
    private double[] inMax = {   72,  72,  72,  72,  72,  72,  72};
    private double[] outMin = {-0.8,-0.8,-0.8,-0.8,-0.8,-0.8,-0.8};
    private double[] outMax = { 0.8, 0.8, 0.8, 0.8, 0.8, 0.8, 0.8};
    private DataNorm dataNorm = new DataNorm(inMin, inMax, outMin, outMax);

    //The input time series.
    private TimeSeries inputTS;
    
    //The input data.
    private Data inputData;
    
    //The output data.
    private Data outputData;
    
    //The output time series.
    private TimeSeries outputTS;
    
    /**
    * Creates a new Anom002Model by loading the neural network
    * weights.
    */
    public Anom003Model() {
        //Create the network.
//        this.createNetwork();
        this(true);
    }
    
    public Anom003Model(boolean readFromFile) {
        super(readFromFile);
        if (readFromFile) this.createNetwork();

        //The name of this model.
        this.modelName = "Anom 003";
    
        //Information about the model.
        this.readInfo("anom003info.txt");
    }
    
    //////////////////////////////////////
    //Private methods.
    //////////////////////////////////////
    
    /**
     *Create the network from the ASCII files. It is assumed that the
     *network has two layers and that the transfer functions for both
     *layers are tanh.
     */
    private void createNetwork() {
        //Create the two layers.
        Layer[] layers = new Layer[2];
        
        //Layer one.
        //Read weights.
        Weights[] weights = WeightsCreator.read(baseDir + "anom003l1.txt");
        //Create neurons, assume tanh.
        TanhNeuron[] neurons = new TanhNeuron[weights.length];
        for (int n=0;n<neurons.length;n++) {
            neurons[n] = new TanhNeuron(weights[n]);
        }
        //Create layer.
        layers[0] = new Layer(neurons);
        
        //Layer two.
        weights = WeightsCreator.read(baseDir + "anom003l2.txt");
        //Create neurons, assume tanh.
        neurons = new TanhNeuron[weights.length];
        for (int n=0;n<neurons.length;n++) {
            neurons[n] = new TanhNeuron(weights[n]);
        }
        //Create layer.
        layers[1] = new Layer(neurons);
        
        //Create network.
        this.network = new Network(layers);
    }
    
    /**
    *When the neural network has been run the output is available
    *as a <code>saaps.ann.Data</code> object. This method creates
    *a <code>TimeSeries</code> object. The time field is obtained
    *from the input time series and the data field is obtained
    *from the output <code>Data</code> object. The data field contain
    *the probability in percent that the predicted event is an anomaly.
    *<p>
    *The output from the network is a number in the interval -0.8
    *to 0.8. If the output is less than zero then we have a "no anomaly"
    *event (0), else we have an "anomaly" event (1). The absolute
    *value of the network output is used to determine the probability
    *that the prediction is correct. The table below gives the probabilies
    *for different outputs.
     * <table cols=2 width=300>
     * <tr>
     * <td>            y &lt -0.400 </td> <td> 11% </td>
     * </tr>
     * <tr>
     * <td> -0.400 &lt y &lt -0.300 </td> <td> 32% </td>
     * </tr>
     * <tr>
     * <td> -0.300 &lt y &lt -0.200 </td> <td> 26% </td>
     * </tr>
     * <tr>
     * <td> -0.200 &lt y &lt -0.100 </td> <td> 32% </td>
     * </tr>
     * <tr>
     * <td> -0.100 &lt y &lt 0.000 </td> <td> 45% </td>
     * </tr>
     * <tr>
     * <td> 0.000 &lt y &lt 0.100 </td> <td> 76% </td>
     * </tr>
     * <tr>
     * <td> 0.100 &lt y &lt 0.200 </td> <td> 61% </td>
     * </tr>
     * <tr>
     * <td> 0.200 &lt y &lt 0.300 </td> <td> 78% </td>
     * </tr>
     * <tr>
     * <td> 0.300 &lt y &lt 0.400 </td> <td> 74% </td>
     * </tr>
     * <tr>
     * <td> 0.400 &lt y           </td> <td> 87% </td>
     * </tr>
     * </table>
    */
    private void createOutputTimeSeries() {
        //Get the output data.
        this.outputData = this.network.getOutput();
//        this.outputData.print();
        //The prediction times.
        //The model is a nowcasting network so the prediction times
        //will be equal to the input times.
        Date[] d = new Date[this.outputData.getNumOfExamples()];
        //The data contain one field: probability for an anomaly.
        double[] data = new double[d.length];
        for (int i=0;i<d.length;i++) {
            //The last date in the output time series
            //shall be equal to the last date in the input time series.
            int j = i - d.length + this.inputTS.getLength();
            d[i] = this.inputTS.getDate(j);
            //Assign the probability for anomaly.
            data[i] = this.getProbability(this.outputData.get(0,i));
        }
        //There is only one output in the output data. This data
        //are the data in the output time series.
        String fieldName = "Probability";
        String fieldUnit = "%";
        outputTS = new TimeSeries(d,data,fieldName,fieldUnit);
    }
    
    /**
     * Return the probability, in percent, that the prediction is
     * an anomaly. The limits and probabilities are
     * <table cols=2 width=300>
     * <tr>
     * <td>            y &lt -0.400 </td> <td> 11% </td>
     * </tr>
     * <tr>
     * <td> -0.400 &lt y &lt -0.300 </td> <td> 32% </td>
     * </tr>
     * <tr>
     * <td> -0.300 &lt y &lt -0.200 </td> <td> 26% </td>
     * </tr>
     * <tr>
     * <td> -0.200 &lt y &lt -0.100 </td> <td> 32% </td>
     * </tr>
     * <tr>
     * <td> -0.100 &lt y &lt 0.000 </td> <td> 45% </td>
     * </tr>
     * <tr>
     * <td> 0.000 &lt y &lt 0.100 </td> <td> 76% </td>
     * </tr>
     * <tr>
     * <td> 0.100 &lt y &lt 0.200 </td> <td> 61% </td>
     * </tr>
     * <tr>
     * <td> 0.200 &lt y &lt 0.300 </td> <td> 78% </td>
     * </tr>
     * <tr>
     * <td> 0.300 &lt y &lt 0.400 </td> <td> 74% </td>
     * </tr>
     * <tr>
     * <td> 0.400 &lt y           </td> <td> 87% </td>
     * </tr>
     * </table>
    */
    private double getProbability(double y) {
	//The upper limits of the network output for each probability.
	double[] limits = {-0.4, -0.3, -0.2, -0.1,  0.0,
	                    0.1,  0.2,  0.3,  0.4,  Double.POSITIVE_INFINITY};
	//The probabilities.
	double[] probs =  {11, 32, 26, 32, 45, 76, 61, 78, 74, 87};
	//Loop as long as the network output is larger
	//than the limits.
	int i = -1;
	while (y>limits[++i]);
	return probs[i];
   }
    
    //////////////////////////////////////
    //Public methods.
    //////////////////////////////////////
    
    /**
     *Read the data for the last 14 days. The network needs seven
     *days of data for the time delay lines. This means that the
     *network will make predictions for the last seven days.
     */
    public void readTimeSeries() {
        //Get current date.
        Date d2 = new Date();
        Date d1 = new Date(d2.getTime() - (15L * 24L * 60L * 60L * 1000L));
        this.readTimeSeries(d1,d2);
    }
    
    /**
     *Read data for the interval given by the two dates. At least seven
     *days of data need to be read as the time delay line is seven days
     *long. If <code>n</code> days are read then the model will produce
     *<code>n-7+1</code> days with predictions.
     *
     *@param d1 The start date
     *@param d2 The end date
     */
    public void readTimeSeries(Date d1, Date d2) {
        //Create the database connection.
//        Database db = Database.getInstance("KpDB",true);
        ODIDatabase db = new ODIDatabase();
        //Get the time series object.
//        try {
//            this.inputTS = db.retrieveTS(d1,d2,true);
            this.inputTS = db.retrieveTS("KPAP_3h",d1,d2);
//            db.close();
//            System.out.println(this.inputTS.getLength());
//        }
//        catch (IOException e) {
//            System.err.println(e);
//        }
        //Find the latest available data.
        int dataIndex = this.inputTS.getLength();
        while (Double.isNaN(this.inputTS.getData(--dataIndex,0))) {
//            System.out.println(dataIndex + " " + this.inputTS.getData(dataIndex,0));
        }
        //Create daily sums with three hour sample interval.
        //Find the number of data points.
        //Each day contain 8 Kp values, thus dataindex - 8 + 1.
        int numOfDays = dataIndex - 7;
        //Create daily sums based on the last available data.
        Date[] sumKpDate = new Date[numOfDays];
        double[] sumKp = new double[numOfDays];
        for (int i=sumKp.length-1;i>=0;i--) {
            //Step over the eight Kp values per day.
            for (int j=dataIndex;j>dataIndex-8;j--) {
//                System.out.println(j + " " + this.inputTS.getData(j,0));
                sumKp[i] += this.inputTS.getData(j,0);
            }
            //Set the sum Kp date to the last date of the sum Kp interval.
            sumKpDate[i] = new Date(this.inputTS.getDate(dataIndex).getTime());
//            System.out.println(i + " " + sumKp[i]);
            //Update the data index to point at the next eight Kp values.
            dataIndex -= 1;
        }
        //Create the time series.
        this.inputTS = new TimeSeries(sumKpDate,sumKp);
//        for (int i=0;i<this.inputTS.getLength();i++) {
//            System.out.println(this.inputTS.getDate(i) + " " + this.inputTS.getData(i,0));
//        }
        //Create the input data.
        this.inputData = this.tsMap.convert(inputTS);
//        this.inputData.print();
        //Normalize the input data.
//        System.out.println(this.dataNorm);
        this.dataNorm.normalize(this.inputData);
//        this.inputData.print();
    }
    
    /**
     *Use this method to set the time for which the model should
     *produce a prediction.
     *
     *@param predDate The prediction time.
     */
    public void setPredictionTime(Date predDate) {
        //This is a nowcasting network, so the prediction date
        //is the same as the input date.
        //Go back 7 days to ensure that 6 full days of data are loaded.
        Date d1 = new Date(predDate.getTime() - (7L * 24L * 60L * 60L * 1000L));
        this.readTimeSeries(d1,predDate);
    }
    
    /**
     *Run the model.
     */
    public void run() {
        this.network.setInput(inputData);
        this.network.run();
    }
    
    /**
    * Get the predicted output. The output will be a <code>TimeSeries</code>
    * object with two data fields.  The two data fields
    *are:
    *<ul>
    *<li> predicted event, 0="no anomaly", 1="anomaly",
    *<li> probability, in percent, that the prediction is correct.
    *</ul>
    *The output from the network is a number in the interval -0.8
    *to 0.8. If the output is less than zero then we have a "no anomaly"
    *event (0), else we have an "anomaly" event (1). The absolute
    *value of the network output is used to determine the probability
    *that the prediction is correct. The table below gives the probabilies
    *for different outputs.
     * <table cols=2 width=300>
     * <tr>
     * <td>            y &lt -0.400 </td> <td> 11% </td>
     * </tr>
     * <tr>
     * <td> -0.400 &lt y &lt -0.300 </td> <td> 32% </td>
     * </tr>
     * <tr>
     * <td> -0.300 &lt y &lt -0.200 </td> <td> 26% </td>
     * </tr>
     * <tr>
     * <td> -0.200 &lt y &lt -0.100 </td> <td> 32% </td>
     * </tr>
     * <tr>
     * <td> -0.100 &lt y &lt 0.000 </td> <td> 45% </td>
     * </tr>
     * <tr>
     * <td> 0.000 &lt y &lt 0.100 </td> <td> 76% </td>
     * </tr>
     * <tr>
     * <td> 0.100 &lt y &lt 0.200 </td> <td> 61% </td>
     * </tr>
     * <tr>
     * <td> 0.200 &lt y &lt 0.300 </td> <td> 78% </td>
     * </tr>
     * <tr>
     * <td> 0.300 &lt y &lt 0.400 </td> <td> 74% </td>
     * </tr>
     * <tr>
     * <td> 0.400 &lt y           </td> <td> 87% </td>
     * </tr>
     * </table>
    *
    *@return The TimeSeries object.
    */
    public TimeSeries getOutput() {
        this.createOutputTimeSeries();
        return this.outputTS;
    }
    
    /**
    * Get a description of the model.
    *
    *@return A String with model information.
    */
    public String toString() {
        return this.network.toString();
    }
    
    public static void main (String args[]) {
        System.out.println("Creating model ...");
        Anom003Model model = new Anom003Model();
        System.out.println(model);
        System.out.println("Reading input data ...");
        java.util.Calendar cal = java.util.Calendar.getInstance(java.util.TimeZone.getTimeZone("GMT"));
        cal.clear();
        cal.set(java.util.Calendar.YEAR,Integer.parseInt(args[0]));
        cal.set(java.util.Calendar.MONTH,Integer.parseInt(args[1])-1);
        cal.set(java.util.Calendar.DATE,Integer.parseInt(args[2]));
        java.util.Date predDate = cal.getTime();
        model.setPredictionTime(predDate);
        System.out.println("Run ...");
        model.run();
        System.out.println("Get output ...");
        saaps.util.TimeSeries ts = model.getOutput();
        for (int i=0;i<ts.getLength();i++) {
            cal.setTime(ts.getDate(i));
            System.out.println(cal.get(java.util.Calendar.YEAR) + " " +
                               ((int) cal.get(java.util.Calendar.MONTH)+1) + " " +
                               cal.get(java.util.Calendar.DATE) + " " +
                               ts.getData(i,0));
        }
    }
}
