package saaps.util;

import java.io.IOException;
import java.util.Date;
import java.util.Vector;
import saaps.dbt.Database;

/**
*
* @author
* @version
*/
public abstract class Average extends Object
{
  /**
  Get a TimeSeries object from the database over the given
  interval and the given sampling rate and time resolution.
  <p>
  The time resolution is defined by the window [w1,w2] over
  which the averaging is made.
  <p>
  Example: Get ACE one minute data with 1 hour central averages.
  <br>
  TimeSeries ts = Average.getFromDB("ACEmagDB",sd,ed,si,w1,w2)
  <br>
  where sd and ed are the start and end dates (Date objects) and
  <ul>
  <li> si = 1000 * 60 * 60, one hour in milliseconds,
  <li> w1 = -1000 * 60 * 30, 1/2 hour back in time,
  <li> w2 = +1000 * 60 * 30, 1/2 hour forward in time.
  </ul>

  @param dboStr The name of the database object
  @param sd The start date
  @param ed The end date
  @param si Sample interval in milliseconds
  @param w1 Start of average window in milliseconds
  @param w2 End of average window in milliseconds
  @param ignoreNaN If true then ignore NaN before averaging
  */
  public static TimeSeries getFromDB(String dboStr, Date sd, Date ed, long si, long w1, long w2, boolean ignoreNaN)
  {
    //Get the start and end times.
    long st = sd.getTime();
    long et = ed.getTime();

    //Get the start and end dates to upload data.
    long stLoad = sd.getTime() + 2*w1;  //Strictly only 1*w1 needed.
    long etLoad = ed.getTime() + 2*w2;  //Strictly only 1*w2 needed.

    //Ensure that there are integer steps from st to et.
    long i = (long) ((et-st)/si + 1L) ;
    et = st + i*si;

    //Get data.
    Database dbo = Database.getInstance(dboStr);
    Vector v = null;
    try {
      v = dbo.retrieve( new Date(stLoad), new Date(etLoad) );
    }
    catch (IOException e)
    {
      System.err.println(e);
    }
    TimeSeries ts = new TimeSeries(v);

    //Initiate the resulting time series object.
    TimeSeries res = new TimeSeries(st,et,si,ts.getNumOfDataFields());

    //Calculate the averages.
    for (int j=0;j<res.getLength();j++)
    {
      double[] tmp = ts.extract(res.getDate(j).getTime()+w1,res.getDate(j).getTime()+w2).average(ignoreNaN);
      res.setData(j,tmp);
    }

    return res;
  }
}
