#! /usr/bin/perl -w
###
# $Id: 02-D-Li.pl,v 1.4 2004/06/23 12:06:39 stamper Exp $
#
#  Purpose
#  
#   To list (part of) a Dataset object.
#   
#  Arguments/Operator input
#  
#   Argument       Description
#   --------------------------
#   $session       A session id
#   $user          The user making the request
#   $source        The source of the request
#   $datasetType   Should always be 'D' for Dataset
#   $datasetName   The name of the dataset
#   $datasetOwner  The owner of the dataset
#   @variables     A list of dataset variables to show
#   $start         The first record to fetch (counting from 1)
#   $end           The last record to fetch
#   $format        The format for the output (ASCII or CDF)
#
#  Returns (on standard output)
#  
#   status.code     0 if by endCGI (success), 1 if by abortCGI (failure)
#   status.message  An appropriate message
#   dataset.type    |
#   dataset.name    | The details of the dataset listed
#   dataset.owner   |
#   listing         The listing of the (projection of) the dataset
#  
#  External routines
#  
#   Sedat.pm       Several utility routines in module Sedat.
#   
#  Description
#   
#   After checking the input details the data file of the requested dataset is
#   located.  In the case of a system dataset (one owned by user SYSTEM), it
#   can be found in the STPDF LRD; all other datasets are actually stored in a
#   file called 'data.cdf' in the directory that represents the dataset object
#   in the database.  
#   
#   The routine get_data is called, passing the details of the dataset file
#   and the list of variables to retrieve.  The dataset name is amended to use
#   the STPDF MULT extension form {n,m} to restrict the range of records to
#   retrieve. The dataset listing is then returned to the calling program.
#   
#   Any failures result in a call to Sedat::abortCGI.
#   
#  Author	Richard Stamper
#
#  $Log: 02-D-Li.pl,v $
#
#  Revision 2.0  2009/10/21 10:29:00  dhc
#  Added support for MySQL to run from ODI backend
#
#  Revision 1.5  2009/05/04 13:57:00  heynderickx
#  Implemented usage of ODI database system for system datasets
#
#  Revision 1.4  2004/06/23 12:06:39  stamper
#  Use an absolute URL when locating the temporary file for listing data,
#  using the CGI-provided SERVER_NAME and SERVER_PORT environment variables.
#
#  Revision 1.3  2001/05/04 14:46:34  stamper
#  Now returns CDF or ASCII format.  To accommodate this, the routine is the
#  only CGI routine in SEDAT that does not use the standard interface
#  protocol.  Instead, it directly returns a 'Location:' header allowing the
#  result file to be fetched directly using the HTTP protocol.
#
#  Revision 1.2  2001/02/27 16:10:30  stamper
#  Allow user datasets to be ASCII or Binary as well as CDF
#
#  Revision 1.1  2000/10/16 15:19:15  stamper
#  List a projection of a dataset.
#
#
###
use CGI qw(:cgi);
use Sedat;
use POSIX;

$remoteuser   = remote_user();
$session      = param('session');
$user         = param('user');
$source       = param('source');
$datasetType  = param('dataset.type');
$datasetName  = param('dataset.name');
$datasetOwner = param('dataset.owner');
@variables    = param('variables');
$start        = param('start');
$end          = param('end');
$format       = param('format');

if ($datasetOwner ne 'SYSTEM') {
# Check for valid connection details
  checkStandard($user,$remoteuser,$session,$source);
}

# Check dataset details and abort if anything is missing or incorrect
errorCGI('Missing type for dataset') if (! defined $datasetType);
errorCGI('Bad type for dataset: '.$datasetType) if ($datasetType ne 'D');
errorCGI('Missing name for dataset') if (! defined $datasetName);
errorCGI('Missing owner for dataset') if (! defined $datasetOwner);

# Locate the dataset, as known to $user
if ($datasetOwner eq 'SYSTEM') {
  $dataset = objectName('SYSTEM', 'SYSTEM', $datasetType, $datasetName);
} else {
  $dataset = objectName(($datasetOwner eq 'SYSTEM') ? 'SYSTEM' : $user,
  		          $datasetOwner,$datasetType,$datasetName);
  errorCGI('No such dataset found') unless (-e $dataset);
}

# System datasets are now accessed via the ODI database
if ($datasetOwner ne 'SYSTEM') {
# User datasets are found in the directory of the dataset object and are
# always called one of 'data.{asc,bin.cdf}' 
    $filename = "data{$start,$end}";
    $filedir  = objectName($user,$datasetOwner,'D',$datasetName);
    @matches = grep { -e joinName($filedir,'data').'.'.$_ } qw/cdf ascp bin/;
    errorCGI('Dataset has invalid data type') if ($#matches < 0);
    $filetype = uc($matches[0]);
}
# Remove all spaces from the @variables list
map {s/ //g} @variables;

# Set the LD_LIBRARY_PATH environment variable to the value of the
# STPDFDYNAMIC variable (needed on Suns)
$ENV{LD_LIBRARY_PATH} = $ENV{STPDFDYNAMIC};

$dtime = time();
$tmpname  = 'tmp-' . $user . $dtime;
$location = joinName('',$dataset,$tmpname);
$basein  = systemParam('OBJECT_DIR');
$baseout = systemParam('WEB_DIR');
$location =~ s!$basein!$baseout!;
$location = join('',
		 'http://',
		 $ENV{SERVER_NAME},':',
		 $ENV{SERVER_PORT},'/',
		 $location);

# Get the dataset output using the appropriate method - 'get_data' for ASCII,
# and 'convert_data' for CDF
if ($datasetOwner eq 'SYSTEM') {
  $userDir = objectName($user,$user,'U',$user);
  $IDL_dir = joinName($userDir,'idl');
  $systemDir = joinName(objectName('SYSTEM','SYSTEM','U','SYSTEM'),'idl');
  $idl_pro = joinName('', $dataset, 'tmp-' . $user . $dtime . '.pro');

  open(my $out, ">",  $idl_pro) or errorCGI('Error creating $idl_pro');
  print $out "!QUIET=1\n";
  print $out "!PATH='${systemDir}:${IDL_dir}:'+!PATH\n";
  print $out ".COMPILE odi_tools\n";

  if ($format eq 'ASCII') {
    print $out "odi_ascii, '" . joinName($dataset,$tmpname) . "', '" . $datasetName . "', ";
    $location .= '.txt';
  } else {
    print $out "odi_cdf, '" . joinName($dataset,$tmpname) . "', '" . $datasetName . "', ";
    $location .= '.cdf';
  }

  print $out "varnames=['" . join("','", @variables) . "'], ";
  print $out "recordrange=[$start,$end]\n";
  print $out "EXIT\n";
  close $out or errorCGI("$out: $!");
  $idl_cmd = systemParam('IDL_RPC');
  $command = $idl_cmd." -quiet ". $idl_pro;
} 
else {
  if ($format eq 'ASCII') {
    $command = "./get_data '$filename' '$filedir' '$filetype' '".
               join(',',@variables)."' > ". joinName($dataset,$tmpname).".txt";
    $location .= ".txt";
  } else {
    $command = "./convert_data '$filename' '$filedir' '$filetype' ".
               "'$tmpname' '$dataset' 'CDF' '".join(',',@variables)."'";
    $location .= ".cdf";
  }
}

#errorCGI("Location: $location");
$rc = system($command);
errorCGI("Failed to convert dataset : rc = $rc\n$command\n") if ($rc != 0);
print "Location: $location\n\n";

# 
# Local abort routine
#
sub errorCGI {
    my ($msg) = @_;

    print "Content-type: text/plain\n\n";
    
    print "ERROR\n\n";
    print $msg,"\n";
    exit(1);
}

#
# checkStandard - Carry out standard checks for data items that all CGI
#                   communications from the SEDAT server should return
sub checkStandard {
    my ($user,$remoteuser,$session,$source) = @_;

    # Check user
    if (! defined $user) {
	errorCGI('Missing user name');
    }
    if (! defined $remoteuser) {
	errorCGI('Missing remote user name');
    }
    if ($remoteuser ne $user) {
	errorCGI('Mismatch between user name and remote user name');
    }

    # Check session
    if (! defined $session) {
    	errorCGI('Missing session id');
    }

    # Check source
    if (! defined $source) {
    	errorCGI('Missing source id');
    }
}
