#! /usr/bin/perl -w
###
# $Id: 02-D-Op.pl,v 1.4 2001/05/04 14:48:14 stamper Exp $
#
#  Purpose
#  
#   To open a Dataset object.
#   
#  Arguments/Operator input
#  
#   Argument       Description
#   --------------------------
#   $session       A session id
#   $user          The user making the request
#   $source        The source of the request
#   $datasetType   Should always be 'D' for Dataset
#   $datasetName   The name of the dataset
#   $datasetOwner  The owner of the dataset
#
#  Returns
#  
#   status.code     0 if by endCGI (success), 1 if by abortCGI (failure)
#   status.message  An appropriate message
#   dataset.type    |
#   dataset.name    | The details of the dataset opened
#   dataset.owner   |
#   description	    The dataset description
#   numents	    The number of entries in the dataset
#   variables	    The names of the variables in the dataset
#  
#  External routines
#  
#   Sedat.pm       Several utility routines in module Sedat.
#   
#  Description
#  
#   After checking the input details the data file of the requested dataset is
#   located.  In the case of a system dataset (one owned by user SYSTEM), it
#   can be found in the STPDF LRD; all other datasets are actually stored in a
#   file called 'data.cdf' in the directory that represents the dataset object
#   in the database.  
#   
#   The description file of the dataset is read and the contents stored.  The
#   routines get_columns and get_numents are then called, passing the details
#   of the dataset file, to retrieve the column names and number of
#   entries. These are then returned to the calling program with the
#   description. 
#   
#   Any failures result in a call to Sedat::abortCGI.
#   
#  Author	Richard Stamper
#
#  $Log: 02-D-Op.pl,v $
#  Revision 2.0  2009/10/21 10:29:00  dhc
#  Added support for MySQL to run from ODI backend
#
#  Revision 1.4  2001/05/04 14:48:14  stamper
#  Check for SYSTEM objects and handle differently from user objects.
#  Use the ASCP type not ASC for ASCII datasets (should be CDF).
#  Set the LD_LIBRARY_PATH variable.
#  Remove blanks from variable names.
#
#  Revision 1.3  2001/02/27 16:10:55  stamper
#  Allow user datasets to be ASCII or Binary as well as CDF.
#
#  Revision 1.2  2000/10/16 15:20:08  stamper
#  Tidied up the dataset details check.
#  Get the number of dataset entries.
#
#
###

use CGI qw(:cgi);
use Sedat;

$remoteuser   = remote_user();
$session      = param('session');
$user         = param('user');
$source       = param('source');
$datasetType  = param('dataset.type');
$datasetName  = param('dataset.name');
$datasetOwner = param('dataset.owner');

print "Content-type: text/plain\n\n";

if ($datasetOwner eq 'SYSTEM') {
# Open a connection to the ODI database
  $dbh = ODI_connect() or abortCGI('Error connecting to ODI database');
}
else {
  $remoteuser   = $user;
# Check for valid connection details
  checkConnection($user,$remoteuser,$session,$source);
}

# Check dataset details and abort if any missing or incorrect
abortCGI('Missing type for dataset') if (! defined $datasetType);
abortCGI('Bad type for dataset: '.$datasetType) if ($datasetType ne 'D');
abortCGI('Missing name for dataset') if (! defined $datasetName);
abortCGI('Missing owner for dataset') if (! defined $datasetOwner);

# If a system dataset use the real name, o/w the name known to the user
if ($datasetOwner eq 'SYSTEM') {
  $dataset = $datasetName;
} else {
  $dataset = objectName($user,$datasetOwner,$datasetType,$datasetName);
  abortCGI('No such dataset found') unless (-e $dataset);
}

# Read the description file and store the contents
if ($datasetOwner eq 'SYSTEM') {
  my $sth = $dbh->prepare("SELECT ds.id, ds.name, ds.numrecs, dm.value FROM dataset AS ds
                           LEFT JOIN dataset_metadata AS dm ON (ds.id=dm.dataset_id)
                           WHERE ds.sedat_name='$dataset' AND
                           dm.attribute='Logical_source_description'");
  $sth->execute();
  if (my $ref = $sth->fetchrow_arrayref()) {
    $dataset_id = $ref->[0];
    $dataset_name = $ref->[1];
    $numents = $ref->[2];
    $description = $ref->[3];
  } else {
    abortCGI('Dataset $dataset not found');
  }
  $sth->finish();
} else {
  open DESC, joinName($dataset,"description") or
    abortCGI('Failed to open dataset description file');
  $description = join('',<DESC>);
  close DESC or
    abortCGI('Failed to close dataset description file');
}

# System datasets are accessed via the database
if ($datasetOwner ne 'SYSTEM') {
# User datasets are found in the directory of the dataset object and are
# always a CDF format file called 'data.cdf'
    $filename = 'data';
    $filedir  = objectName($user,$datasetOwner,'D',$datasetName);
    @matches = grep { -e joinName($filedir,'data').'.'.$_ } qw/cdf ascp bin/;
    abortCGI('Dataset has invalid data type') if ($#matches < 0);
    $filetype = uc($matches[0]);
}

# Set the LD_LIBRARY_PATH environment variable to the value of the
# STPDFDYNAMIC variable (needed on Suns)
$ENV{LD_LIBRARY_PATH} = $ENV{STPDFDYNAMIC};

# Get the variables using the routine 'get_columns' in the CGI directory
# The grep removes any diagnostic line generated by chp_getdbno
if ($datasetOwner eq 'SYSTEM') {
  my $sth = $dbh->prepare("SELECT name FROM variable_metadata
                           WHERE dataset_id=$dataset_id");
  $sth->execute() or abortCGI($dbh->errstr);
  while (my $ref = $sth->fetchrow_arrayref()) {
    push @variables, $ref->[0];
  }
  $sth->finish();
} else {
    @variables = grep !/^in/,`./get_columns '$filename' '$filedir' '$filetype'`;
}
map {chomp;} @variables;
map {s/\s*$//} @variables;

# Get the number of entries using the similar routine 'get_numents'
if ($datasetOwner ne 'SYSTEM') {
  $numents = `./get_numents '$filename' '$filedir' '$filetype'`;
}
chomp $numents;

# Close the connection to the ODI database
if ($datasetOwner eq 'SYSTEM') {
  $dbh->disconnect();
}

# Return the retrieved information
returnField("dataset.type","T","D");
returnField("dataset.name","T",$datasetName);
returnField("dataset.owner","T",$datasetOwner);
returnField("description","T",$description);
returnField("numents","I",$numents);
returnField("variables","T",@variables);
endCGI("Dataset $datasetOwner/$datasetName opened");

